"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = multipart;

var _multiparty = require("multiparty");

var _multiparty2 = _interopRequireDefault(_multiparty);

var _fs = require("fs");

var _fs2 = _interopRequireDefault(_fs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @param  {Object}	原始信息
 * @return {FileInfo}	文件信息
 */
function createFile(info, saveFile, getUrl, getPath) {
  //字段名称，文件名，临时存储路径，文件大小，文件类型
  const name = info.fieldName,
        filename = info.originalFilename,
        path = info.path,
        size = info.size,
        mime = info.headers && info.headers["content-type"] || ""; //是否已销毁

  let destroy = false;
  let save;
  const file = {
    //获取字段名称
    get name() {
      return destroy ? '' : name;
    },

    //获取原始文件名
    get filename() {
      return destroy ? '' : filename;
    },

    //获取文件尺寸
    get size() {
      return destroy ? 0 : size;
    },

    //获取文件类型
    get mime() {
      return destroy ? '' : mime;
    },

    //获取文件临时路径
    get path() {
      return destroy ? '' : path;
    },

    //获取文件二进制信息
    async getBuffer() {
      return destroy ? null : new Promise((a, b) => _fs2.default.readFile(path, (e, d) => e ? b(e) : a(d)));
    },

    //获取文件输入流
    getReadStream(options) {
      return destroy ? null : _fs2.default.createReadStream(path, options);
    },

    //销毁文件
    destroy() {
      if (destroy) {
        return;
      }

      destroy = true;

      _fs2.default.unlink(path, e => e);
    },

    async save(...p) {
      if (save) {
        return save;
      }

      if (destroy) {
        return '';
      }

      return save = saveFile(this, ...p);
    },

    async getUrl(...p) {
      return destroy ? '' : getUrl((await file.save(...p)));
    },

    async getPath(...p) {
      return destroy ? '' : getPath((await file.save(...p)));
    }

  };
  return file;
}

function getFileList(files, saveFile, getUrl, getPath) {
  const list = [];

  for (let k in files) {
    files[k].forEach(file => list.push(createFile(file, saveFile, getUrl, getPath)));
  }

  return list;
}

function getBody(fields) {
  const data = {};

  for (let k in fields) {
    const d = fields[k];
    data[k] = d.length === 1 ? d[0] : d;
  }

  return data;
}
/**
 * 获取请求文件
 */


async function multipart(req, saveFile, getUrl, getPath) {
  req.files = [];

  if (req.method !== 'POST') {
    return;
  }

  return new Promise(next => new _multiparty2.default.Form().parse(req, function (err, fields, files) {
    if (err) {
      return next();
    }

    req.body = getBody(fields);
    req.files = getFileList(files, saveFile, getUrl, getPath);
    return next();
  }));
}